@@ -5,6 +5,7 @@ Calculate timeseries data per image.
 
 from concurrent import futures
 from datetime import datetime
+import dateutil.parser
 import logging
 import math
 import multiprocessing
@@ -69,6 +70,7 @@ def calc_stats_per_image(
         # Loop over all images to start the data preparation for each of them in parallel...
         image_paths.sort()
         start_time = datetime.now()
+        esaSwitchedProcessingMethod = dateutil.parser.isoparse('2021-02-23').timestamp()
         nb_todo = len(image_paths)
         nb_errors_max = 10
         nb_errors = 0
@@ -112,9 +114,12 @@ def calc_stats_per_image(
                         nb_todo -= 1
                         continue
 
+                    # Fast-24h <- 2021-02-23 -> NRT-3H
+                    productTimelinessCategory = 'Fast-24h' if dateutil.parser.isoparse(image_info['acquisition_date']).timestamp() < esaSwitchedProcessingMethod else 'NRT-3h'
+                    
                     # If sentinel1 and wrong productTimelinessCategory, skip: we only want 1 type to evade images used twice
-                    if image_info['satellite'].startswith('S1') and image_info['productTimelinessCategory'] != 'Fast-24h':
-                        logger.info(f"SKIP image, productTimelinessCategory should be 'Fast-24h', but is: {image_info['productTimelinessCategory']} for {image_path_str}")
+                    if image_info['satellite'].startswith('S1') and image_info['productTimelinessCategory'] != productTimelinessCategory:
+                        logger.info(f"SKIP image, productTimelinessCategory should be '{productTimelinessCategory}', but is: {image_info['productTimelinessCategory']} for {image_path_str}")
                         nb_todo -= 1
                         continue
 
@@ -498,8 +503,12 @@ def prepare_calc(
     if 'footprint' in image_info:
         logger.info(f"poly: {image_info['footprint']['shape']}")
         footprint_shape = image_info['footprint']['shape']
+
+    if image_info['image_epsg'] == 'NONE':
+        raise Exception(f'target_epsg == NONE: {image_info}')
+
     features_gdf = load_features_file(
-            features_filepath=features_filepath, 
+            features_filepath=features_filepath,
             target_epsg=image_info['image_epsg'],
             columns_to_retain=[id_column, 'geometry'],
             bbox=image_info['image_bounds'],
@@ -581,7 +590,7 @@ def load_features_file(
 
     # Determine the correct filename for the input features in the correct projection.
     if features_epsg != target_epsg:
-        features_prepr_filepath = features_filepath.parent / f"{features_filepath.stem}_{target_epsg}.gpkg"
+        features_prepr_filepath = features_filepath.parent / f"{features_filepath.stem}_{target_epsg:.0f}.gpkg"
     else:
         features_prepr_filepath = features_filepath
 
@@ -952,6 +961,11 @@ def get_image_info(image_path: Path) -> dict:
                 y, x = coord.split(',') 
                 linear_ring.append((float(x), float(y)))
             image_info['footprint']['shape'] = sh_polygon.Polygon(linear_ring)
+
+            # get epsg
+            epsg = metadata_root.find('imageProjection/EPSG').text
+            image_info['image_epsg'] = float(epsg)
+            image_info['image_crs'] = 'EPSG:' + epsg
         except Exception as ex:
             raise Exception(f"Exception extracting info from {metadata_xml_filepath}") from ex
 
@@ -983,6 +997,7 @@ def get_image_info(image_path: Path) -> dict:
 
                 image_info['instrument_mode'] = manifest_root.find("metadataSection/metadataObject/metadataWrap/xmlData/safe:platform/safe:instrument/safe:extension/s1sarl1:instrumentMode/s1sarl1:mode", ns).text
                 image_info['orbit_properties_pass'] = manifest_root.find("metadataSection/metadataObject/metadataWrap/xmlData/safe:orbitReference/safe:extension/s1:orbitProperties/s1:pass", ns).text
+                image_info['acquisition_date'] = manifest_root.find("metadataSection/metadataObject/metadataWrap/xmlData/safe:acquisitionPeriod/safe:startTime", ns).text
 
                 # Now have a look in the files themselves to get band info,...
                 # TODO: probably cleaner/easier to read from metadata files...
@@ -1014,8 +1029,9 @@ def get_image_info(image_path: Path) -> dict:
                         with rasterio.open(str(band_filepath)) as src:
                             image_info['image_bounds'] = src.bounds
                             image_info['image_affine'] = src.transform
-                            image_info['image_crs'] = str(src.crs)
-                            image_info['image_epsg'] = image_info['image_crs'].upper().replace('EPSG:', '')
+                            if src.crs is not None:
+                                image_info['image_crs'] = src.crs.to_string()
+                                image_info['image_epsg'] = src.crs.to_epsg()
                         #logger.debug(f"Image metadata read: {image_info}")
                     if band_filepath.stem == 'Gamma0_VH':
                         band = 'VH'
@@ -1039,15 +1055,15 @@ def get_image_info(image_path: Path) -> dict:
             # Now parse the first .safe file
             # TODO: maybe check if the info in all safe files  are the same or?.?
             manifest_xml_filepath = manifest_xml_filepaths[0]
-            
+
             try:
                 manifest = ET.parse(str(manifest_xml_filepath))
                 manifest_root = manifest.getroot()
 
                 # Define namespaces...
                 ns = {"safe": "http://www.esa.int/safe/sentinel-1.0",
-                      "s1": "http://www.esa.int/safe/sentinel-1.0/sentinel-1",
-                      "s1sarl1": "http://www.esa.int/safe/sentinel-1.0/sentinel-1/sar/level-1"}
+                    "s1": "http://www.esa.int/safe/sentinel-1.0/sentinel-1",
+                    "s1sarl1": "http://www.esa.int/safe/sentinel-1.0/sentinel-1/sar/level-1"}
 
                 #logger.debug(f"Parse manifest info from {metadata_xml_filepath}")
                 image_info["transmitter_receiver_polarisation"] = []
@@ -1056,12 +1072,17 @@ def get_image_info(image_path: Path) -> dict:
                 image_info['productTimelinessCategory'] = manifest_root.find("metadataSection/metadataObject/metadataWrap/xmlData/s1sarl1:standAloneProductInformation/s1sarl1:productTimelinessCategory", ns).text
                 image_info["instrument_mode"] = manifest_root.find("metadataSection/metadataObject/metadataWrap/xmlData/safe:platform/safe:instrument/safe:extension/s1sarl1:instrumentMode/s1sarl1:mode", ns).text
                 image_info["orbit_properties_pass"] = manifest_root.find("metadataSection/metadataObject/metadataWrap/xmlData/safe:orbitReference/safe:extension/s1:orbitProperties/s1:pass", ns).text
+                image_info['acquisition_date'] = manifest_root.find("metadataSection/metadataObject/metadataWrap/xmlData/safe:acquisitionPeriod/safe:startTime", ns).text
 
                 # Now have a look in the files themselves to get band info,...
                 # TODO: probably cleaner/easier to read from metadata files...
                 # For coherence filename, remove extra .LCO1 extension + date
                 #image_datafilebasename, _ = os.path.splitext(image_basename_noext)
                 #image_datafilebasename = image_datafilebasename[:image_datafilebasename.rindex('_')]
+                
+                metadata = ET.parse(str(image_path / 'metadata.xml'))
+                metadata_root = metadata.getroot()
+
                 image_basename_noext_nodate = image_basename_noext[:image_basename_noext.rfind('_', 0)]
                 image_datafilename = f"{image_basename_noext_nodate}_byte.tif"
                 image_datafilepath = image_path / image_datafilename
@@ -1070,9 +1091,9 @@ def get_image_info(image_path: Path) -> dict:
                 with rasterio.open(str(image_datafilepath)) as src:
                     image_info['image_bounds'] = src.bounds
                     image_info['image_affine'] = src.transform
-                    image_info['image_crs'] = str(src.crs)
-                    image_info['image_epsg'] = image_info['image_crs'].upper().replace('EPSG:', '')
-                #logger.debug(f"Image metadata read: {image_info}")
+                    if src.crs is not None:
+                        image_info['image_crs'] = src.crs.to_string()
+                        image_info['image_epsg'] = src.crs.to_epsg()
 
                 # Add specific info about the bands
                 image_info["bands"] = {}
@@ -1129,15 +1150,29 @@ def get_image_info(image_path: Path) -> dict:
 
             #logger.debug(f"Parse metadata info from {metadata_xml_filepath}")
             image_info['Cloud_Coverage_Assessment'] = float(metadata_root.find("n1:Quality_Indicators_Info/Cloud_Coverage_Assessment", ns).text)
+            image_info['acquisition_date'] = metadata_root.find("n1:General_Info/Product_Info/PRODUCT_START_TIME", ns).text
             
         except Exception as ex:
             raise Exception(f"Exception extracting info from {metadata_xml_filepath}") from ex
             
         # Now have a look in the files themselves to get band info,...
-        # TODO: probably cleaner/easier to read from metadata files?
         image_datadir = image_path / "GRANULE"
         band_filepaths = list(image_datadir.rglob(f"*.jp2"))
 
+        metadata_filepath = list(image_datadir.rglob("MTD_TL.xml"))[0]
+        try:
+            # read epsg
+            metadata = ET.parse(str(metadata_filepath))
+            metadata_root = metadata.getroot()
+
+            ns = {'n1': "https://psd-14.sentinel2.eo.esa.int/PSD/S2_PDI_Level-2A_Tile_Metadata.xsd",
+                  'xsi': "http://www.w3.org/2001/XMLSchema-instance"}
+
+            image_info['image_crs'] = metadata_root.find('n1:Geometric_Info/Tile_Geocoding/HORIZONTAL_CS_CODE', ns).text
+            image_info['image_epsg'] = float(image_info['image_crs'].replace('EPSG:', ''))
+        except Exception as ex:
+            raise Exception(f"Exception extracting info from {metadata_filepath}") from ex
+
         # If no files were found, error!
         if len(band_filepaths) == 0:
             message = f"No image files found in {image_datadir}!"
@@ -1175,21 +1210,25 @@ def get_image_info(image_path: Path) -> dict:
             logger.debug(f"Read image metadata from {band_filepath}")
             with rasterio.open(str(band_filepath)) as src:
                 image_info['bands'][band]['bounds'] = src.bounds
-                image_info['bands'][band]['affine'] = src.transform
-                image_info['bands'][band]['crs'] = str(src.crs)
-                image_info['bands'][band]['epsg'] = str(src.crs).upper().replace('EPSG:', '')
-                
-                # Store the crs also on image level, and check if all bands have the same crs 
-                if i == 0:
-                    image_info['image_bounds'] = image_info['bands'][band]['bounds']
-                    image_info['image_crs'] = image_info['bands'][band]['crs']
-                    image_info['image_epsg'] = image_info['bands'][band]['epsg']
+                image_info['bands'][band]['affine'] = src.transform                
+                if src.crs is None:
+                    image_info['bands'][band]['crs'] = image_info['image_crs']
+                    image_info['bands'][band]['epsg'] = image_info['image_epsg']
                 else:
-                    if(image_info['image_crs'] != image_info['bands'][band]['crs'] 
-                       or image_info['image_epsg'] != image_info['bands'][band]['epsg']):
-                        message = f"Not all bands have the same crs for {image_info}"
-                        logger.error(message)
-                        raise Exception(message)
+                    image_info['bands'][band]['crs'] = src.crs.to_string()
+                    image_info['bands'][band]['epsg'] = src.crs.to_epsg()
+                
+            # Store the crs also on image level, and check if all bands have the same crs 
+            if i == 0:
+                image_info['image_bounds'] = image_info['bands'][band]['bounds']
+                #image_info['image_crs'] = image_info['bands'][band]['crs']
+                #image_info['image_epsg'] = image_info['bands'][band]['epsg']
+            else:
+                if(image_info['image_crs'] != image_info['bands'][band]['crs'] 
+                    or image_info['image_epsg'] != image_info['bands'][band]['epsg']):                       
+                    message = f"Not all bands have the same crs for {image_info}"
+                    logger.error(message)
+                    raise Exception(message)
                 
         logger.debug(f"Image metadata read: {image_info}")
 
